@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2011, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,12 +24,14 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
 #include <string.h>
 #include <grp.h>
 #include <sys/ioctl.h>
 #include <linux/vzcalluser.h>
 #include <sys/personality.h>
 #include <linux/reboot.h>
+#include <sys/mount.h>
 
 #include "vzerror.h"
 #include "res.h"
@@ -42,10 +44,12 @@
 #include "iptables.h"
 #include "readelf.h"
 #include "vzsyscalls.h"
+#include "cpt.h"
 
 #define ENVRETRY	3
 
-static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode);
+static int env_stop(vps_handler *h, envid_t veid, const char *root,
+		int stop_mode);
 
 static inline int setluid(uid_t uid)
 {
@@ -58,9 +62,9 @@ static int set_personality(unsigned long mask)
 	unsigned long per;
 
 	per = personality(0xffffffff) | mask;
-	logger(3, 0, "Set personality %#10.8x", per);
+	logger(3, 0, "Set personality %#10.8lx", per);
 	if (personality(per) == -1) {
-		logger(2, errno, "Unable to set personality PER_LINUX32");
+		logger(-1, errno, "Unable to set personality PER_LINUX32");
 		return  -1;
 	}
 	return 0;
@@ -74,7 +78,7 @@ static int set_personality32()
 }
 #endif
 
-int vz_env_create_data_ioctl(vps_handler *h,
+static int vz_env_create_data_ioctl(vps_handler *h,
 	struct vzctl_env_create_data *data)
 {
 	int errcode;
@@ -176,8 +180,7 @@ vps_handler *vz_open(envid_t veid)
 	return h;
 
 err:
-	if (h != NULL)
-		free(h);
+	free(h);
 	if (vzfd != -1)
 		close(vzfd);
 	if (stdfd != -1)
@@ -227,7 +230,7 @@ int vps_is_run(vps_handler *h, envid_t veid)
  * @param		CT root
  * @return		0 on success
  */
-int vz_chroot(char *root)
+int vz_chroot(const char *root)
 {
 	int i;
 	sigset_t sigset;
@@ -270,32 +273,40 @@ int vz_setluid(envid_t veid)
 	return 0;
 }
 
-/*
- * Checks if sysfs needs to be enabled for this CT.
- * Now we do that only for distributions from sysfs_dists.
- * FIXME: provide a generic way to enable/disable sysfs per CT.
-*/
-static int sysfs_required(vps_res *res)
+static int vz_env_configure(int fd, envid_t veid, const char *osrelease)
 {
-	static char *sysfs_dists[] = {"opensuse", "suse", "sles", NULL};
+	int ret = 0;
+	struct vzctl_ve_configure *cparam;
+	int len;
 
-	tmpl_param *tmp = &res->tmpl;
-	int len, i;
-	char *name;
+	len = strlen(osrelease) + 1;
+	cparam = calloc(1, sizeof(struct vzctl_ve_configure) + len);
+	if (cparam == NULL)
+		return VZ_RESOURCE_ERROR;
 
-	for (i = 0; name = sysfs_dists[i], name != NULL; i++) {
-		len = strlen(name);
-		if (tmp->ostmpl != NULL &&
-			!strncmp(tmp->ostmpl, name, len))
-		{
-			return 1;
-		}
-		if (tmp->dist != NULL &&
-			!strncmp(tmp->dist, name, len))
-		{
-			return 1;
-		}
-	}
+	cparam->veid = veid;
+	cparam->key = VE_CONFIGURE_OS_RELEASE;
+	cparam->size = len;
+	strcpy(cparam->data, osrelease);
+
+	if (ioctl(fd, VZCTL_VE_CONFIGURE, cparam) != 0)
+		if (errno != ENOTTY)
+			ret = VZ_SET_OSRELEASE;
+
+	free(cparam);
+
+	return ret;
+}
+
+static int configure_sysctl()
+{
+	int fd;
+
+	fd = open("/proc/sys/net/ipv6/conf/all/forwarding", O_WRONLY);
+	if (fd == -1)
+		return -1;
+	write(fd, "0", 1);
+	close(fd);
 	return 0;
 }
 
@@ -312,7 +323,9 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	res = (vps_res *) data;
 	memset(&create_param, 0, sizeof(create_param));
 	create_param.iptables_mask = get_ipt_mask(res->env.ipt_mask);
-	logger(3, 0, "Set iptables mask %#10.8x", create_param.iptables_mask);
+	logger(3, 0, "Set iptables mask %#10.8llx",
+			(unsigned long long) create_param.iptables_mask);
+	clean_hardlink_dir("/");
 	if (res->cpu.vcpus != NULL)
 		create_param.total_vcpus = *res->cpu.vcpus;
 	env_create_data.veid = veid;
@@ -323,12 +336,12 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 
 	create_param.feature_mask = res->env.features_mask;
 	create_param.known_features = res->env.features_known;
-	if (!(res->env.features_known & VE_FEATURE_SYSFS) &&
-			sysfs_required(res)) {
+	/* sysfs enabled by default, unless explicitly disabled */
+	if (! (res->env.features_known & VE_FEATURE_SYSFS)) {
 		create_param.feature_mask |= VE_FEATURE_SYSFS;
 		create_param.known_features |= VE_FEATURE_SYSFS;
 	}
-	logger(3, 0, "Set features mask %016Lx/%016Lx",
+	logger(3, 0, "Set features mask %016llx/%016llx",
 			create_param.feature_mask,
 			create_param.known_features);
 
@@ -342,7 +355,7 @@ try:
 			case EINVAL:
 				ret = VZ_ENVCREATE_ERROR;
 				/* Run-time kernel did not understand the
-				 * latest create_parem -- so retry with
+				 * latest create_param -- so retry with
 				 * the old env_create_param structs.
 				 */
 				switch (env_create_data.datalen) {
@@ -371,20 +384,38 @@ try:
 		}
 		goto env_err;
 	}
+	if (res->env.osrelease != NULL) {
+		ret = vz_env_configure(h->vzfd, veid,
+				res->env.osrelease);
+		if (ret != 0)
+			goto env_err;
+	}
+
 	close(h->vzfd);
 	/* Create /fastboot to skip run fsck */
 	fd = open("/fastboot", O_CREAT | O_RDONLY, 0644);
 	close(fd);
 
-	mk_reboot_script();
-	if (res->dq.ugidlimit != NULL)
-		mk_quota_link();
 	if (res->misc.wait == YES) {
 		if (add_reach_runlevel_mark()) {
 			ret = VZ_WAIT_FAILED;
 			goto env_err;
 		}
 	}
+
+	mount("proc", "/proc", "proc", 0, 0);
+	if (stat_file("/sys"))
+		mount("sysfs", "/sys", "sysfs", 0, 0);
+
+	if (create_param.feature_mask & VE_FEATURE_NFSD) {
+		mount("nfsd", "/proc/fs/nfsd", "nfsd", 0, 0);
+		make_dir("/var/lib/nfs/rpc_pipefs", 1);
+		mount("sunrpc", "/var/lib/nfs/rpc_pipefs", "rpc_pipefs", 0, 0);
+	}
+	configure_sysctl();
+	if (res->dq.ugidlimit != NULL)
+		mk_quota_link();
+
 	/* Close status descriptor to report that
 	 * environment is created.
 	*/
@@ -392,7 +423,7 @@ try:
 	/* Now we wait until CT setup will be done
 	   If no error, then start init, otherwise exit.
 	*/
-	if (read(wait_p, &ret, sizeof(ret)) != 0)
+	if (read(wait_p, &ret, sizeof(ret)) == 0)
 		return 0;
 	if ((fd = open("/dev/null", O_RDWR)) != -1) {
 		dup2(fd, 0);
@@ -411,7 +442,7 @@ env_err:
 }
 
 static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
-	int wait_p, int err_p, env_create_FN fn, void *data)
+	int wait_p, int old_wait_p, int err_p, env_create_FN fn, void *data)
 {
 	int ret, pid;
 
@@ -426,12 +457,12 @@ static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		logger(-1, errno, "Unable to fork");
 		return VZ_RESOURCE_ERROR;
 	} else if (pid == 0) {
-		if ((ret = vps_set_cap(veid, &res->cap)))
+		if ((ret = vps_set_cap(veid, &res->env, &res->cap)))
 			goto env_err;
 		if (fn == NULL) {
 			ret = _env_create(h, veid, wait_p, err_p, (void *)res);
 		} else {
-			ret = fn(h, veid, wait_p, err_p, data);
+			ret = fn(h, veid, wait_p, old_wait_p, err_p, data);
 		}
 env_err:
 		if (ret)
@@ -442,9 +473,11 @@ env_err:
 }
 
 int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
-	int wait_p[2], int err_p[2], env_create_FN fn, void *data)
+		int wait_p[2], int old_wait_p[2], int err_p[2],
+				env_create_FN fn, void *data)
 {
 	int ret, pid, errcode;
+	int old_wait_fd;
 	int status_p[2];
 	struct sigaction act, actold;
 
@@ -460,6 +493,8 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	act.sa_flags = SA_NOCLDSTOP;
 	sigaction(SIGCHLD, &act, NULL);
 
+	get_osrelease(res);
+
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Can not fork");
 		ret =  VZ_RESOURCE_ERROR;
@@ -473,9 +508,15 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		close(err_p[0]);
 		fcntl(wait_p[0], F_SETFD, FD_CLOEXEC);
 		close(wait_p[1]);
+		if (old_wait_p) {
+			fcntl(old_wait_p[0], F_SETFD, FD_CLOEXEC);
+			close(old_wait_p[1]);
+			old_wait_fd = old_wait_p[0];
+		} else
+			old_wait_fd = -1;
 
-		ret = vz_real_env_create(h, veid, res, wait_p[0], err_p[1], fn,
-			data);
+		ret = vz_real_env_create(h, veid, res, wait_p[0],
+					old_wait_fd, err_p[1], fn, data);
 		if (ret)
 			write(STDIN_FILENO, &ret, sizeof(ret));
 		exit(ret);
@@ -483,9 +524,11 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	/* Wait for environment created */
 	close(status_p[1]);
 	close(wait_p[0]);
+	if (old_wait_p)
+		close(old_wait_p[0]);
 	close(err_p[1]);
 	ret = read(status_p[0], &errcode, sizeof(errcode));
-	if (ret) {
+	if (ret > 0) {
 		ret = errcode;
 		switch(ret) {
 		case VZ_NO_ACCES:
@@ -506,6 +549,10 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 			logger(0, 0, "Unable to set"
 				" wait functionality");
 			break;
+		case VZ_SET_OSRELEASE:
+			logger(-1, 0, "Unable to set osrelease to %s",
+					res->env.osrelease);
+			break;
 		}
 	}
 err:
@@ -524,7 +571,7 @@ static void fix_numiptent(ub_param *ub)
 		return;
 	min_ipt = min_ul(ub->numiptent[0], ub->numiptent[1]);
 	if (min_ipt < MIN_NUMIPTENT) {
-		logger(-1, 0, "Warning: NUMIPTENT %d:%d is less"
+		logger(-1, 0, "Warning: NUMIPTENT %lu:%lu is less"
 			" than minimally allowable value, set to %d:%d",
 			ub->numiptent[0], ub->numiptent[1],
 			MIN_NUMIPTENT, MIN_NUMIPTENT);
@@ -541,11 +588,96 @@ static void fix_cpu(cpu_param *cpu)
 	}
 }
 
+#define SBIN_INIT "/sbin/init"
+#define UPSTART_INIT "/lib/upstart/init"
+#define UPSTART "upstart"
+#define SYSTEMD "systemd"
+#define FEDORA_RELEASE "/etc/fedora-release"
+#define DEV_CONSOLE "/dev/console"
+#define DEV_NULL "/dev/null"
+
+static int fix_ve_systemd(const char *root)
+{
+	struct stat st;
+	char file[STR_SIZE];
+	char dest[STR_SIZE];
+
+	/* Check if init is systemd */
+	snprintf(file, STR_SIZE - 1, "%s/" SBIN_INIT, root);
+	if (lstat(file, &st) != 0)
+		return 0; /* Some error -- giving up */
+	if (!S_ISLNK(st.st_mode))
+		return 0; /* Not a symlink, so not systemd */
+	if (readlink(file, dest, STR_SIZE - 1) < 0)
+		return 0; /* Some error -- giving up */
+	if (strstr(dest, SYSTEMD) == NULL)
+		return 0; /* Not systemd */
+
+	logger(2, 0, "systemd init found, changing to upstart");
+	snprintf(dest, STR_SIZE - 1, "%s/" UPSTART_INIT, root);
+	if (stat(dest, &st) != 0)
+		goto err;
+	if (unlink(file) != 0)
+		goto err;
+	if (symlink(UPSTART_INIT, file) != 0)
+		goto err;
+
+	return 0;
+err:
+	logger(-1, errno, "Failed to set upstart as init");
+	return -1;
+}
+
+static int fix_ve_devconsole(const char *root)
+{
+	char file[STR_SIZE];
+	char dest[STR_SIZE];
+	struct stat st;
+
+	snprintf(file, STR_SIZE - 1, "%s/" DEV_CONSOLE , root);
+	if (lstat(file, &st) == 0 && S_ISLNK(st.st_mode))
+		return 0; /* already fixed */
+
+	/* Check that it's Fedora */
+	snprintf(file, STR_SIZE - 1, "%s/" FEDORA_RELEASE , root);
+	if (stat(file, &st) != 0)
+		return 0; /* not Fedora */
+
+	/* Check if init is upstart */
+	snprintf(file, STR_SIZE - 1, "%s/" SBIN_INIT , root);
+	if (lstat(file, &st) != 0)
+		return 0; /* Some error -- giving up */
+	if (!S_ISLNK(st.st_mode))
+		return 0; /* Not a symlink, so not upstart */
+	if (readlink(file, dest, STR_SIZE - 1) < 0)
+		return 0; /* Some error -- giving up */
+	if (strstr(dest, UPSTART) == NULL)
+		return 0; /* Not upstart */
+
+	/* For upstart 1.2 to work in CT,
+	 * /dev/console should point to /dev/null
+	 */
+	snprintf(file, STR_SIZE - 1, "%s/" DEV_CONSOLE, root);
+	if (unlink(file) != 0)
+		if (errno != ENOENT) {
+			logger(-1, errno, "Failed to remove %s", file);
+			return -1;
+		}
+
+	if (symlink(DEV_NULL, file) != 0) {
+		logger(-1, errno, "Failed to symlink %s -> " DEV_NULL, file);
+		return -1;
+	}
+
+	return 0;
+}
+
 int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	skipFlags skip, struct mod_action *mod,
 	env_create_FN fn, void *data)
 {
 	int wait_p[2];
+	int old_wait_p[2];
 	int err_p[2];
 	int ret, err;
 	char buf[64];
@@ -565,8 +697,7 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 		return ret;
 	dist_name = get_dist_name(&res->tmpl);
 	ret = read_dist_actions(dist_name, DIST_DIR, &actions);
-	if (dist_name != NULL)
-		free(dist_name);
+	free(dist_name);
 	if (ret)
 		return ret;
 	logger(0, 0, "Starting container ...");
@@ -576,15 +707,34 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	}
 	if (!vps_is_mounted(res->fs.root)) {
 		/* increase quota to perform setup */
-		quouta_inc(&res->dq, 100);
+		quota_inc(&res->dq, 100);
 		if ((ret = vps_mount(h, veid, &res->fs, &res->dq, skip)))
 			return ret;
-		quouta_inc(&res->dq, -100);
+		quota_inc(&res->dq, -100);
 	}
+	/* Fedora 14/15 hacks */
+	if (fix_ve_devconsole(res->fs.root) != 0)
+		return VZ_FS_BAD_TMPL;
+	if (fix_ve_systemd(res->fs.root) != 0)
+		return VZ_FS_BAD_TMPL;
+
 	if (pipe(wait_p) < 0) {
 		logger(-1, errno, "Can not create pipe");
 		return VZ_RESOURCE_ERROR;
 	}
+	/* old_wait_p is needed for backward compatibility with older kernels,
+	 * while for recent ones (that support CPT_SET_LOCKFD2) we use wait_p.
+	 *
+	 * If old_wait_p is closed without writing any data, it's "OK to go"
+	 * signal, and if data are received from old_wait_p it's "no go"
+	 * signal". Note that such thing doesn't work if vzctl segfaults,
+	 * because in this case the descriptor will be closed without
+	 * sending data.
+	 */
+	if (pipe(old_wait_p) < 0) {
+		logger(-1, errno, "Can not create pipe");
+		return VZ_RESOURCE_ERROR;
+	}
 	if (pipe(err_p) < 0) {
 		close(wait_p[0]);
 		close(wait_p[1]);
@@ -597,8 +747,12 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	sigaction(SIGPIPE, &act, NULL);
 	fix_numiptent(&res->ub);
 	fix_cpu(&res->cpu);
-	if ((ret = vz_env_create(h, veid, res, wait_p, err_p, fn, data)))
+
+	ret = vz_env_create(h, veid, res, wait_p,
+				old_wait_p, err_p, fn, data);
+	if (ret)
 		goto err;
+
 	if ((ret = vps_setup_res(h, veid, &actions, &res->fs, param,
 		STATE_STARTING, skip, mod)))
 	{
@@ -616,15 +770,25 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 			}
 		}
 	}
-	/* Close fd to start /sbin/init */
-	if (close(wait_p[1]))
-		logger(-1, errno, "Unable to close fd to start init");
+	/* Tell the child that it's time to start /sbin/init */
+	err = 0;
+	if (write(wait_p[1], &err, sizeof(err)) != sizeof(err))
+		logger(-1, errno, "Unable to write to waitfd to start init");
+	close(wait_p[1]);
+	close(old_wait_p[1]);
 err:
 	free_dist_actions(&actions);
 	if (ret) {
 		/* Kill environment */
-		logger(-1, 0, "Container start failed");
-		write(wait_p[1], &err, sizeof(err));
+		logger(-1, 0, "Container start failed (try to check kernel "
+				"messages, e.g. \"dmesg | tail\")");
+		/* Close wait fd without writing anything to it
+		 * to signal the child that we have failed to configure
+		 * the environment, so it should not start /sbin/init
+		 */
+		close(wait_p[1]);
+		write(old_wait_p[1], &err, sizeof(err));
+		close(old_wait_p[1]);
 	} else {
 		if (!read(err_p[0], &ret, sizeof(ret))) {
 			if (res->misc.wait == YES) {
@@ -682,7 +846,7 @@ int vps_start(vps_handler *h, envid_t veid, vps_param *param,
 	return vps_start_custom(h, veid, param, skip, mod, NULL, NULL);
 }
 
-static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
+static int real_env_stop(vps_handler *h, envid_t veid, const char *vps_root,
 	int stop_mode)
 {
 	int ret;
@@ -723,7 +887,8 @@ static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
 	return 0;
 }
 
-static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
+static int env_stop(vps_handler *h, envid_t veid, const char *root,
+		int stop_mode)
 {
 	struct sigaction act, actold;
 	int i, pid, ret = 0;
@@ -820,39 +985,12 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	if ((ret = env_stop(h, veid, res->fs.root, stop_mode)))
 		goto end;
 	mod_cleanup(h, veid, action, param);
-	vps_cleanup_res(h, veid, param, STATE_STOPPING);
+	/* Cleanup CT IPs */
+	run_net_script(veid, DEL, &param->del_res.net.ip,
+			STATE_STOPPING, param->res.net.skip_arpdetect);
 	ret = vps_umount(h, veid, res->fs.root, skip);
-	/* Clear CT network configuration */
-	ret = run_pre_script(veid, VPS_STOP);
 
 end:
 	free_str_param(&param->del_res.net.ip);
 	return ret;
 }
-
-/** Restart CT.
- *
- * @param h		CT handler.
- * @param veid		CT ID.
- * @param param		CT parameters.
- * @return		0 on success.
- */
-int vps_restart(vps_handler *h, envid_t veid, vps_param *param)
-{
-	int ret;
-
-	logger(0, 0, "Restarting container");
-	if (vps_is_run(h, veid) &&
-		(ret = vps_stop(h, veid, param, M_REBOOT, SKIP_NONE, NULL)))
-	{
-		return ret;
-	}
-	if (param->opt.start_disabled == YES)
-	{
-		logger(-1, 0, "Container start disabled in config");
-		return VZ_VE_START_DISABLED;
-	}
-	ret = vps_start(h, veid, param, SKIP_NONE, NULL);
-
-	return ret;
-}
