@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,11 +28,14 @@
 #include "types.h"
 #include "config.h"
 
+#include "logger.h"
+#include "util.h"
+
 /* #include <linux/magic.h>
  * Instead of including a file which is not present in all environments,
  * we copy-paste reiserfs magic from it
  */
-#define REISERFS_SUPER_MAGIC    0x52654973
+#define REISERFS_SUPER_MAGIC	0x52654973
 
 #define SYSRSRV		52428800
 #define MEMPERVE	5542912
@@ -43,7 +46,6 @@
 
 #define PROCMEM		"/proc/meminfo"
 #define PROCTHREADS	"/proc/sys/kernel/threads-max"
-#define PROCCPU		"/proc/cpuinfo"
 
 #define MAX_SL		3
 
@@ -68,7 +70,7 @@
 #define PTY_PVE		512
 
 /* default diskspace values */
-#define HOST_DS		10737418240ULL /* 10 GB */
+#define HOST_DS		10485760ULL /* 10 GB */
 #define DEF_DS		225280
 #define HOST_DI		100000
 #define DEF_DI		88000
@@ -152,7 +154,7 @@ struct par_limits params[NUMUBC];
 /* Global variables */
 unsigned long long mem_total, low_total, swap_total, ds_total, di_total;
 long pagesize, proc_calc;
-int num_ve, ve_allowed, osl;
+int num_ve, ve_allowed, osl, vswap;
 
 float	k_kmem[MAX_SL]		= {1, 1.2, 1.8};
 float	k_nproc[MAX_SL]		= {1, 1.5, 2};
@@ -166,31 +168,33 @@ float	k_privvm[MAX_SL]	= {6, 3, 1.5};
 int	k_pglock[MAX_SL]	= {10, 3, 1};
 int	k_msl[MAX_SL]		= {10485760, 2097152, 0};
 
-char *level_string[MAX_SL+1] = {
-"Free resource distribution. Any parameters may be increased\0",
-"Normal resource distribution. Secondary parameters may be increased\0",
-"Partial resource shortage. Auxiliary parameters may be increased\0",
-"Overall resource shortage. Please, do not change any parameters!\0"
+char *level_string[MAX_SL + 1] = {
+	"Free resource distribution. Any parameters may be increased",
+	"Normal resource distribution. Secondary parameters may be increased",
+	"Partial resource shortage. Auxiliary parameters may be increased",
+	"Overall resource shortage. Please, do not change any parameters!"
 };
 
-void usage(int rc)
+static void usage(int rc)
 {
-	fprintf(rc ? stderr : stdout, "Usage: vzsplit [-f config_name] "
-			"| [-n numves] | [-s swap_size]\n"
-		"\t -f specified config name\n"
-		"\t -n specified number of containers\n"
-		"\t -s specified swap in Kbytes\n");
+	fprintf(rc ? stderr : stdout,
+"Usage: vzsplit [-f config_name] [-n num_CTs] [-s swap_size] [-v yes|no]\n"
+"	-f		configuration sample name\n"
+"	-n		number of containers\n"
+"	-s		swap space, Kbytes\n"
+"	-v yes|no	generate VSwap config (overrides auto-detection)\n"
+);
 	exit(rc);
 }
 
-void header(FILE *fp)
+static void header(FILE *fp)
 {
 	fprintf(fp,
-		"# Configuration file generated by vzsplit for %d containers\n"
-		"# on HN with total amount of physical mem %llu Mb\n"
-		"# low memory %llu Mb, swap size %llu Mb, Max treads %lu\n"
-		"# Resourse commit level %d:\n# %s\n",
-		num_ve,
+"# Configuration file generated by vzsplit for %d container%c\n"
+"# on HN with total amount of physical memory %llu Mb,\n"
+"# low memory %llu Mb, swap size %llu Mb, max threads %lu.\n"
+"# Resource commit level %d:\n# %s\n",
+		num_ve, (num_ve == 1)? ' ': 's',
 		(mem_total >> 20),
 		(low_total >> 20), (swap_total >> 20), proc_calc,
 		osl, level_string[osl]
@@ -198,37 +202,20 @@ void header(FILE *fp)
 	return;
 }
 
-int get_cpupower(int *cpuunits)
-{
-	FILE *fd;
-	char str[1024];
-	int val, total = 0;
+#define PRINT_PARAM(i) \
+	fprintf(fp, "%s=\"%s\"\n", \
+		ubcnames[i], ubcstr( \
+			(unsigned long) params[i].bar, \
+			(unsigned long) params[i].lim) )
 
-	*cpuunits = DEF_CPUUNITS;
-	if ((fd = fopen(PROCCPU, "r")) == NULL) {
-		fprintf(stderr, "Cannot open " PROCCPU "\n");
-		return 1;
-	}
-	while (fgets(str, sizeof(str), fd))
-		if (sscanf(str, "bogomips\t: %u", &val) == 1)
-			total += val;
-	if (total) {
-		total *= 25;
-		*cpuunits = total / (num_ve + 1);
-		return 0;
-	}
-	return 1;
-}
-
-int lconv(char *name)
+static int lconv(char *name)
 {
-	int i, cpuunits;
+	int i;
 	FILE *fp;
 
 	if (name != NULL) {
 		if ((fp = fopen(name, "w")) == NULL) {
-			fprintf(stderr, "Unable to create %s: %s\n", name,
-					strerror(errno));
+			logger(-1, errno, "Unable to create %s", name);
 			return 1;
 		}
 	} else {
@@ -236,40 +223,74 @@ int lconv(char *name)
 	}
 	header(fp);
 
-	fprintf(fp, "# Primary parameters\n");
-	for (i = 0;  i < NUMUBC; i++) {
-		if (i == KMEM)
-			fprintf(fp, "\n# Secondary parameters\n");
-		else if (i ==LOCKPG)
-			fprintf(fp, "\n# Auxiliary parameters\n");
-		fprintf(fp, "%s=\"%lu:%lu\"\n", ubcnames[i],
-				(unsigned long) params[i].bar,
-				(unsigned long)	params[i].lim);
+	if (vswap)
+	{
+		unsigned long mem = params[PRIVVMPG].bar;
+		unsigned long kmem = mem * pagesize / 2;
+
+		fprintf(fp, "\n# This is VSwap-enabled configuration\n\n");
+		fprintf(fp, "PHYSPAGES=\"0:%lu\"\n", mem);
+		fprintf(fp, "SWAPPAGES=\"0:%lu\"\n", mem * 2);
+		fprintf(fp, "KMEMSIZE=\"%lu:%lu\"\n",
+				(unsigned long)(kmem/KMEM_DELTA), kmem);
+		fprintf(fp, "DCACHESIZE=\"%lu:%lu\"\n",
+				(unsigned long)(kmem/2./KMEM_DELTA), kmem/2);
+		fprintf(fp, "LOCKEDPAGES=\"%lu\"\n", mem / 2);
+		fprintf(fp,	"\n"
+				"PRIVVMPAGES=\"unlimited\"\n"
+				"SHMPAGES=\"unlimited\"\n"
+				"NUMPROC=\"unlimited\"\n"
+				"VMGUARPAGES=\"0:unlimited\"\n"
+				"OOMGUARPAGES=\"0:unlimited\"\n"
+				"NUMTCPSOCK=\"unlimited\"\n"
+				"NUMFLOCK=\"unlimited\"\n"
+				"NUMPTY=\"unlimited\"\n"
+				"NUMSIGINFO=\"unlimited\"\n"
+				"TCPSNDBUF=\"unlimited\"\n"
+				"TCPRCVBUF=\"unlimited\"\n"
+				"OTHERSOCKBUF=\"unlimited\"\n"
+				"DGRAMRCVBUF=\"unlimited\"\n"
+				"NUMOTHERSOCK=\"unlimited\"\n"
+				"NUMFILE=\"unlimited\"\n"
+				"NUMIPTENT=\"unlimited\"\n"
+				"\n");
+		PRINT_PARAM(DISKSPACE);
+		PRINT_PARAM(DISKINODES);
 	}
-	get_cpupower(&cpuunits);
-	fprintf(fp, "CPUUNITS=\"%d\"\n", cpuunits);
+	else {
+		fprintf(fp, "# Primary parameters\n");
+		for (i = 0;  i < NUMUBC; i++) {
+			if (i == KMEM)
+				fprintf(fp, "\n# Secondary parameters\n");
+			else if (i ==LOCKPG)
+				fprintf(fp, "\n# Auxiliary parameters\n");
+
+			PRINT_PARAM(i);
+		}
+	}
+	fprintf(fp, "CPUUNITS=\"%d\"\n", DEF_CPUUNITS);
 	if (name) {
 		fclose(fp);
-		fprintf(stderr, "Config %s was created\n", name);
+		logger(-1, 0, "Config %s was created", name);
 	}
 	return 0;
 }
 
-int calculate_values()
+static int calculate_values()
 {
 	long long rest, delta, low_pve, tot_pve;
 	long long kmem, numproc, avnproc, sockbuf, tcpbuf, iptent, numfile;
 	long long dcache, numflock;
 	long long lockedpages, numpty, guarpg, privvm, shmpg, di_pve, ds_pve;
-	int sl = 0;
+	int sl;
 
-	avnproc = tcpbuf = 0;
+	sl = avnproc = tcpbuf = 0;
 	tot_pve = (mem_total + swap_total - SYSRSRV) / num_ve;
 	for (osl = 0; osl < MAX_SL; osl++) {
 		numproc = k_nproc[osl] * proc_calc / num_ve;
 		low_pve = low_total * 0.4 * k_kmem[osl] / num_ve;
 		guarpg = (tot_pve - low_pve) / pagesize;
-		if  (guarpg < MIN_GUARPG) {
+		if (guarpg < MIN_GUARPG) {
 			guarpg = MIN_GUARPG;
 			low_pve = tot_pve - guarpg * pagesize;
 		}
@@ -282,7 +303,7 @@ int calculate_values()
 		if (avnproc < MINPROC / 2)
 			continue;
 
-		if (numproc <  avnproc)
+		if (numproc < avnproc)
 			numproc = avnproc;
 		if ((numproc < 2 * avnproc) && (osl < MAX_SL))
 			numproc = 2 * avnproc;
@@ -303,7 +324,7 @@ int calculate_values()
 		numproc = k_nproc[sl] * proc_calc / num_ve;
 		low_pve = low_total * 0.4 * K_KMEM_MAX / num_ve;
 		guarpg = (tot_pve - low_pve) / pagesize;
-		if  (guarpg < MIN_GUARPG) {
+		if (guarpg < MIN_GUARPG) {
 			guarpg = MIN_GUARPG;
 			low_pve = tot_pve - guarpg * pagesize;
 		}
@@ -316,11 +337,11 @@ int calculate_values()
 			sockbuf = 2 * (MINDGRAMBUF + MINTCPBUF + delta);
 			kmem = low_pve - sockbuf;
 			rest = kmem - avnproc * k_kpp[sl];
-		} while  ((rest < 0) && (numproc > MINPROC));
+		} while ((rest < 0) && (numproc > MINPROC));
 
 		if (rest < 0) {
-			fprintf(stderr, "Fatal resource shortage, try to "
-					"decrease the number of containers\n");
+			logger(-1, 0, "Fatal resource shortage, try to "
+					"decrease the number of containers");
 			return 1;
 		}
 		params[DGRAM].lim = params[DGRAM].bar = MINDGRAMBUF;
@@ -389,7 +410,9 @@ int calculate_values()
 	params[NPHYPG].bar = 0; params[NPHYPG].lim = MAXVAL;
 
 	privvm = guarpg * k_privvm[sl];
-	if (privvm > PRIVVM_PVE * mem_total) {
+	if (privvm > tot_pve / pagesize)
+		privvm = guarpg;
+	if (num_ve > 1 && privvm > PRIVVM_PVE * mem_total) {
 		privvm = PRIVVM_PVE * mem_total;
 		if (guarpg > privvm)
 			guarpg = privvm;
@@ -417,7 +440,7 @@ int calculate_values()
 	return 0;
 }
 
-char * get_ve_private()
+static char * get_ve_private()
 {
 	vps_param *param;
 	char *ve_private, *veidp, *ret;
@@ -427,8 +450,7 @@ char * get_ve_private()
 	vps_parse_config(0, GLOBAL_CFG, param, NULL);
 	ve_private = param->res.fs.private_orig;
 
-	if (ve_private == NULL)
-	{
+	if (ve_private == NULL) {
 		free_vps_param(param);
 		return NULL;
 	}
@@ -445,7 +467,7 @@ char * get_ve_private()
 	return ret;
 }
 
-int check_disk_space() {
+static int check_disk_space() {
 	char * ve_private;
 	int nofs = 0, noinodes = 0;
 	long ve_ds, ve_di;
@@ -454,25 +476,27 @@ int check_disk_space() {
 
 	ve_private = get_ve_private();
 	if (ve_private == NULL) {
-		fprintf(stderr, "WARNING: unable to get VE_PRIVATE value "
-				"from " GLOBAL_CFG ".\n");
+		logger(-1, 0, "WARNING: unable to get VE_PRIVATE value "
+				"from " GLOBAL_CFG ".");
 		nofs = 1;
 	}
 	else {
-		if (statfs (ve_private, &statfs_buf) < 0) {
-			fprintf(stderr, "WARNING: statfs on %s failed: %s.\n",
-				ve_private, strerror(errno));
+		if (statfs(ve_private, &statfs_buf) < 0) {
+			logger(-1, errno, "WARNING: statfs on %s failed",
+				ve_private);
 			nofs = 1;
 		}
 	}
 
 	if (nofs == 1) {
-		fprintf(stderr, "Default disk space values to be used.\n\n");
+		logger(-1, 0, "Default disk space values to be used.\n");
 		ds_total = 0; di_total = 0;
 		return retval;
 	}
 
 	ds_total = statfs_buf.f_blocks;
+	/* convert to Kbytes */
+	ds_total *= (statfs_buf.f_bsize / 1024);
 	di_total = statfs_buf.f_files;
 
 	if (statfs_buf.f_type == REISERFS_SUPER_MAGIC) {
@@ -481,11 +505,11 @@ int check_disk_space() {
 		noinodes = 1;
 	}
 
-	if (ds_total / 2 < HOST_DS / statfs_buf.f_bsize) {
+	if (ds_total / 2 < HOST_DS) {
 		rec = 1;
 		ds_total /= 2;
 	} else
-		ds_total -= HOST_DS / statfs_buf.f_bsize;
+		ds_total -= HOST_DS;
 
 	if (noinodes != 1) {
 		if (di_total / 2 < HOST_DI) {
@@ -495,12 +519,12 @@ int check_disk_space() {
 			di_total -= HOST_DI;
 	}
 	if (rec)
-		fprintf(stderr, "WARNING: Recommended minimal size "
-				"of partition holding %s is 20Gb!\n",
-				ve_private);
+		logger(-1, 0, "WARNING: Recommended minimal size "
+				"of partition holding %s is %llu Gb!",
+				ve_private, HOST_DS / 1024 / 1024 * 2);
 
-	ve_ds = ds_total / (DEF_DS * 1024 / statfs_buf.f_bsize);
-	ve_di = di_total / (DEF_DI);
+	ve_ds = ds_total / DEF_DS;
+	ve_di = di_total / DEF_DI;
 
 	if (ve_ds < num_ve) {
 		retval = 1;
@@ -513,12 +537,12 @@ int check_disk_space() {
 			ve_allowed = ve_di;
 	}
 	if (retval == 1) {
-		fprintf(stderr, "WARNING: partition holding %s do not "
+		logger(-1, 0, "WARNING: partition holding %s do not "
 				"have space required for %d containers\n"
-				"The maximum allowed value is %d\n",
+				"The maximum allowed value is %d",
 				ve_private, num_ve, ve_allowed);
-		fprintf(stderr, "Default disk space values "
-				"will be used\n\n");
+		logger(-1, 0, "Default disk space values "
+				"will be used\n");
 		ds_total = 0; di_total = 0;
 	}
 
@@ -545,15 +569,35 @@ int main(int argc, char **argv)
 	low_total = 0;
 	swap_total = 0;
 
-	while ((opt = getopt(argc, argv, "f:n:s:h")) > 0) {
+	init_log(NULL, 0, 1, 0, 0, "vzsplit");
+
+	vswap = is_vswap_mode();
+
+	while ((opt = getopt(argc, argv, "f:n:s:h:v:")) > 0) {
 		switch(opt) {
+		case 'v':
+			switch (yesno2id(optarg)) {
+				case YES:
+					vswap = 1;
+					break;
+				case NO:
+					vswap = 0;
+					break;
+				default:
+					logger(-1, 0, "Invalid argument "
+							"for -v: %s", optarg);
+					usage(1);
+			}
+			break;
 		case 'f':
 			len = strlen(optarg) + strlen(VPS_CONF_DIR) +
 				strlen("ve-.conf-sample");
-			name = (char*)malloc(len + 1);
+			name = (char *)vz_malloc(len + 1);
+			if (!name)
+				exit(1);
 			sprintf(name, VPS_CONF_DIR "ve-%s.conf-sample", optarg);
 			if (!stat(name, &st)) {
-				fprintf(stderr,"File %s already exist\n",
+				logger(-1, 0, "File %s already exist",
 					name);
 				exit(1);
 			}
@@ -561,23 +605,23 @@ int main(int argc, char **argv)
 		case 'n':
 			num_ve = strtol(optarg, &tail, 10);
 			if (*tail != '\0') {
-				fprintf(stderr, "Invalid argument "
-						"for -n: %s\n",	optarg);
-				exit(1);
+				logger(-1, 0, "Invalid argument "
+						"for -n: %s", optarg);
+				usage(1);
 			}
 			break;
 		case 's':
 			if (optarg[0] == '-') {
-				fprintf(stderr, "Negative value for -s\n");
-				exit(-1);
+				logger(-1, 0, "Negative value for -s");
+				usage(1);
 			}
 			swp = 1;
 			swap_total = strtoll(optarg, &tail, 10);
 			swap_total <<= 10;
 			if (*tail != '\0') {
-				fprintf(stderr, "Invalid argument "
-						"for -s: %s\n",	optarg);
-				exit(1);
+				logger(-1, 0, "Invalid argument "
+						"for -s: %s", optarg);
+				usage(1);
 			}
 			break;
 		case 'h':
@@ -592,17 +636,17 @@ int main(int argc, char **argv)
 	if (num_ve == -1) {
 		printf("Enter the number of containers: ");
 		if (scanf("%d", &num_ve) != 1) {
-			fprintf(stderr, "Invalid value.\n");
+			logger(-1, 0, "Invalid value.");
 			exit(1);
 		}
 	}
 	if (num_ve < 1) {
-		fprintf(stderr, "Incorrect value for number of containers.\n");
+		logger(-1, 0, "Incorrect value for number of containers.");
 		exit(1);
 	}
 
 	if ((fd = fopen(PROCMEM, "r")) == NULL) {
-		fprintf(stderr, "Cannot open " PROCMEM"\n");
+		logger(-1, errno, "Cannot open " PROCMEM);
 		exit(1);
 	}
 
@@ -618,21 +662,22 @@ int main(int argc, char **argv)
 	if (low_total == 0)
 		low_total = mem_total;
 	if (mem_total < SYSRSRV) {
-		fprintf(stderr, "At least 128 Mb of RAM should be "
-				"installed on Hardware Node\n");
+		logger(-1, 0, "At least %d Mb of RAM should be "
+				"installed on Hardware Node",
+				SYSRSRV / 1024 / 1024);
 		exit(1);
 	}
 
 	if (swap_total > 2 * mem_total)
-		fprintf(stderr, "The optimal swap space size is %llu Mb, "
-				"twice bigger than the RAM size\n\n",
+		logger(-1, 0, "The optimal swap space size is %llu Mb, "
+				"twice bigger than the RAM size\n",
 				(2 * mem_total) >> 20);
 	ve_allowed = num_ve;
 	retval = 0;
 
 	if (((mem_total + swap_total - SYSRSRV) / num_ve) < MEMPERVE) {
-		fprintf(stderr, "On node with %llu Mb of memory (RAM + swap) "
-				"%d containers can not be allocated\n",
+		logger(-1, 0, "On node with %llu Mb of memory (RAM + swap) "
+				"%d containers can not be allocated",
 				(mem_total + swap_total) >> 20, num_ve);
 		ve_allowed = (mem_total + swap_total - SYSRSRV) / MEMPERVE;
 		retval = 1;
@@ -641,8 +686,8 @@ int main(int argc, char **argv)
 	if (((low_total - SYSRSRV)/ ve_allowed) < LOWPERVE) {
 		int ve_low;
 
-		fprintf(stderr, "On node with %llu Mb of Low Memory "
-				"%d containers can not be allocated\n",
+		logger(-1, 0, "On node with %llu Mb of Low Memory "
+				"%d containers can not be allocated",
 				low_total >> 20, num_ve);
 		ve_low = (low_total - SYSRSRV) / LOWPERVE;
 		if (ve_low < ve_allowed)
@@ -651,13 +696,13 @@ int main(int argc, char **argv)
 	}
 
 	if (retval != 0) {
-		fprintf(stderr, "The maximum allowed value is %d\n",
+		logger(-1, 0, "The maximum allowed value is %d",
 				ve_allowed);
 		exit(retval);
 	}
 
 	if ((fd = fopen(PROCTHREADS, "r")) == NULL) {
-		fprintf(stderr, "Cannot open " PROCTHREADS "\n");
+		logger(-1, errno, "Cannot open " PROCTHREADS);
 		exit(1);
 	}
 	if (fgets(str, sizeof(str), fd))
@@ -670,7 +715,7 @@ int main(int argc, char **argv)
 	if ((pagesize = sysconf(_SC_PAGE_SIZE)) == -1)
 		pagesize = 4096;
 
-	if (low_total > 2 << 30) {
+	if (low_total > 2U << 30) {
 		if (proc_calc > 2 * MAX_TOTAL_PIDS)
 			proc_calc = 2 * MAX_TOTAL_PIDS;
 	} else if (proc_calc > MAX_TOTAL_PIDS)
@@ -682,4 +727,3 @@ int main(int argc, char **argv)
 	retval = lconv(name);
 	exit(retval);
 }
-
